#include "vxp202.h"
#include <stdio.h>
#include <time.h>
#include <string.h>

#define VxP202_
/* #define DOS
 */
#ifdef DOS
  #include <dos.h>
#endif

/* for now no interrupt driven I2C IO */

/* times in ns */
#define I2C_DLY_FALL    300  /* maximum falling time */
#define I2C_DLY_RAISE  1000  /* maximum raise time */
#define I2C_DLY_BUF    4700  /* bus free time between start and stop */
#define I2C_DLY_HDSTA  4000  /* hold time start period */
#define I2C_DLY_SUSTO  4000  /* setup time stop condition */
#define I2C_DLY_SUDAT   250  /* data setup time */
#define I2C_DLY_LOW    4700  /* low period of SCL clock */
#define I2C_DLY_HIGH   4000  /* high period of SCL clock */

static float I2C_DLY;
static int I2C_PORT;

/* delay for dly ns */
void I2CWait(unsigned int dly) {
  unsigned long i,n;

  n=(unsigned long)(I2C_DLY*(float)dly)+1;
  for (i=0;i<n;i++);
}

int I2CGetSDA() {
  return I2C_PORT & VxP_I2C_DATA;
}

int I2CGetSCL() {
  return I2C_PORT & VxP_I2C_CLK;
}

//#define DEBUG

void I2CProtocol() {
  int sda,scl;

  I2CWait(I2C_DLY_RAISE);
  sda=I2CGetSDA();
  scl=I2CGetSCL();
  if (scl)
    printf("  ");
  else
    printf("    ");
  if (sda)
    printf("  ");
  else
    printf("    ");
  printf("%02x\n",VxP_PortRead(VxP_I2C_CTRL));
}

void I2CSetSDA(int lh) {
  #ifdef VxP202_
    if (lh) {
      I2C_PORT = I2C_PORT | VxP_I2C_DATA;
    } else {
      I2C_PORT = I2C_PORT & VxP_I2C_CLK;
    }
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    #ifdef DEBUG
      I2CProtocol();
    #endif
  #endif
}

void I2CSetSCL(int lh) {
  #ifdef VxP202_
    if (lh) {
      I2C_PORT = I2C_PORT | VxP_I2C_CLK;
    } else {
      I2C_PORT = I2C_PORT & VxP_I2C_DATA;
    }
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    #ifdef DEBUG
      I2CProtocol();
    #endif
  #endif
}

void I2CStart() {
  /*    ____
            |_____
        __________
                    CLCK
  */
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_HDSTA);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
  /* now ready to transmit data */
}

void I2CStop() {
  /*
          __
     ____|
     _______
             CLCK
  */
  /* we assume that both SDA and SCL are low */
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_SUSTO);
  I2CSetSDA(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_BUF);
  /* transmission completed */
}

void I2CHigh() {
  /*
      _______ H
              L
         _
      __| |__ CLK
  */
  /* we assume that both SDA and SCL are low */
  I2CSetSDA(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_HIGH);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL);
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
}

void I2CLow() {
  /*
              H
      _______ L
         _
      __| |__ CLK
  */
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_HIGH);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
}

int I2CAck() {
  int ack,value;
  /*
      _________ H
                L
         ___
      __|   |__ CLK

          ^
          | read ACK / data
  */
  I2CSetSDA(1);
  I2CWait(I2C_DLY_FALL+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE);
  value=VxP_PortRead(VxP_I2C_CTRL);
//  ack=I2CGetSDA();
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL);
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
  printf(" %02x ",value);
  return value;
}

int I2CSendByte(int data) {
  int i;

  for (i=0;i<8;i++) {
    if (data & (1 << i))
      I2CHigh();
    else
      I2CLow();
  }
  i=I2CAck();
  if (!i)
    return 0;
  return 1;
}

static int I2CReadByte(void) {
  int i;
  int data=0;

  /*
      _________ H
                L
         ___
      __|   |__ CLK

          ^
          | read data
  */
  /* set data output bit to H, CLK L */
  return data;
}

void I2CInit() {
  clock_t t1,t2,dt;
  unsigned long i,cnt;

  #ifdef VxP202_
    VxP_PortWrite(0x37,0);
    I2C_PORT = VxP_I2C_CLK | VxP_I2C_DATA | VxP_I2C_FRAME;
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    I2CWait(2*I2C_DLY_BUF);
  #endif

  cnt=1;
  do {
    cnt=cnt*2;
    t1=clock();
    for (i=0;i<cnt;i++);
    t2=clock();
    dt=t2-t1;
  } while (dt<20);
  printf("dt=%d cnt=%f\n",dt,(float)cnt);
  #ifdef DOS
    I2C_DLY=((float)cnt/1e9)*((float)CLK_TCK/(float)dt);
    printf("I2C_DLY=%f\n",I2C_DLY);
  #else
  #endif
}

static void scan_i2c(void) {
  int i;
  for (i=0; i<256; i++) {
    I2CStart();
    if (I2CSendByte(i))
      printf(" ACK\n");
    else
      printf(" NAK\n");
    I2CStop();
  }
}

void scani2c(int port) {
  I2CStart();
  if (I2CSendByte(port))
    printf("Ack\n");
  else
    printf("NAK\n");
  I2CStop();
}

static void set_freq(unsigned short freq) {
  unsigned char config=0x01;
  unsigned short div;
  
  if (freq < 467*16) 
    config = 0x04;
  if (freq < 144*16) 
    config = 0x02;

  div=((freq+16*38.9));
  
  div&=0x7fff;
 /* cli(); /* keep Videotext driver from interfering */
  I2CStart();
  I2CSendByte(0xc2);
  I2CSendByte(freq>>8);
  I2CSendByte(freq&0xff);
  I2CSendByte(0x8e);
  I2CSendByte(config);
  I2CStop();
  /* sti();*/
}

main() {
  int reg,data,sub,freq;
  int l,r,t,b;

  printf("/*\n");
  /* set base to 500h, mem win to C800h with size 32kByte */
  VxP_Initialize(0x500,0xC800,32);
  I2CInit();

  scan_i2c();
  printf("* /\n");
}
