/* 
   linux/drivers/char/vxp202.c
   Copyright (C) 1996 Nils Schmeisser
   Generic Vxp202 driver for Linux running on an IBM compatible computer.
*/

#include "vproc.h"
#include "release.h"

#ifndef word
  typedef unsigned short word;
#endif
#ifndef byte
  typedef unsigned char byte;
#endif

static int VPROC;
static word PORT_SEEK;

#ifdef VxP202
  #include "vxp202.h"
#endif

/*--- the VxP202 ----------------------------------------------*/

static word active_port;

void VxP_SetPort(word port) {
  outw_p(port,VxP_INDEX);
  active_port=port;
}

void VxP_WriteData(word data) {
  outb_p((byte)(data % 256),VxP_DATA_WRITE);
  if (active_port>0x3fff)
    outb_p((byte)((data >> 8) & 0xff),VxP_DATA_WRITE+1);
}

word VxP_ReadData() {
  byte l,h;

  l=inb_p(VxP_DATA_READ  );
  if (active_port>0x3fff)
    h=inb_p(VxP_DATA_READ+1);
  else
    h=0;
  return ((word)h << 8) + (word)l;
}

word old_input_cfg_a;

void VxP_InputPipelineLock() {
  VxP_SetPort(VxP_INPUT_CFG_A);
  old_input_cfg_a=VxP_ReadData();
  VxP_SetPort(VxP_INPUT_CFG_A);
  VxP_WriteData(VxP_INPUT_PIPE_RESET);
}

void VxP_InputPipelineUnlock() {
  VxP_SetPort(VxP_INPUT_CFG_A);
  VxP_WriteData(old_input_cfg_a /*& !VxP_INPUT_PIPE_RESET*/);
}

word VxP_PortRead(word port) {
  VxP_SetPort(port);
  return VxP_ReadData();
}

void VxP_PortWrite(word port,word data) {
  VxP_InputPipelineLock();
  VxP_SetPort(port);
  VxP_WriteData(data);
  VxP_InputPipelineUnlock();
}

/*-------------------------------------------------------------*/

inline int ControlVproc(int cmd,int device) {
  return 0;
}

static int vproc_ioctl (struct inode *inode,
			     struct file *file,
			     unsigned int cmd,
			     unsigned long arg) {
  unsigned int minor;

  #ifdef DEBUG
    printk("Vproc ioctl\n");
  #endif

  if (MAJOR (inode->i_rdev) != VPROC_MAJOR)
    return -EINVAL;
  minor=MINOR(inode->i_rdev);
  if (minor >= VPROC_MAX)
    return -ENODEV;
  return ControlVproc(cmd,minor);
}

static int vproc_open(struct inode *inode, struct file *file) {
  char *code="AURAV1";
  int i;

  unsigned int minor=MINOR(inode->i_rdev);

  if (minor>=VPROC_MAX)
    return -ENODEV;	 /*check for /dev/vproc */
  MOD_INC_USE_COUNT;
  switch (minor) {
    case 0: VPROC=CHIPVxP202; 
            printk("Video-processor type: VxP202\n");
            /*
	      VxP_PortWrite(0x10,0xff);
	      for (i=0;i<6;i++)
	      VxP_PortWrite(0x80,code[i]);
	      VxP_PortWrite(0x80,0xf4);
	      VxP_PortWrite(0x80,0x0b);
	      VxP_PortWrite(0x80,0x40);
	    */
            break;
  }
  VPROC=CHIPVxP202;
  PORT_SEEK=0;
  return 0;
}

static void vproc_release (struct inode *inode, struct file *file) {
  inode->i_atime=CURRENT_TIME;
  MOD_DEC_USE_COUNT;
}

static int vproc_lseek(struct inode *inode,
		       struct file *file,off_t ofs,int whence) {
  PORT_SEEK=(word)ofs;
  return PORT_SEEK;
}

static int vproc_write (struct inode *inode,
			struct file *file,const char *buf, int count) {
  byte data;

  verify_area (VERIFY_READ,(void*)buf,sizeof(byte));
  data=get_fs_byte(buf);

  switch (VPROC) {
    case CHIPVxP202: VxP_PortWrite(PORT_SEEK,data); break;
  }

  return (int)VxP_PortRead(PORT_SEEK);
}

static int vproc_read (struct inode *inode,
		       struct file *file, char *buf, int count) {
  byte data;

  verify_area (VERIFY_WRITE,(void*)buf,sizeof(byte));
  
  data=0;
  switch (VPROC) {
    case CHIPVxP202: data=VxP_PortRead(PORT_SEEK); break;
  }

  put_fs_byte(data,buf);

  return (int)data;
}

static struct file_operations vproc_fops =
{
	vproc_lseek,		/* lseek*/
	vproc_read,
	vproc_write,	
	NULL,			/* readaddr*/
	NULL,			/* select */
	vproc_ioctl,	
	NULL,			/* mmap */
	vproc_open,	
	vproc_release,     
	NULL			/* sync */
};

#ifdef __cplusplus
extern "C" {
#endif

int init_module(void) {

  if (register_chrdev(VPROC_MAJOR,"vproc",&vproc_fops)) {
    printk ("Unable to get major=%d for vproc\n",VPROC_MAJOR);
    return 1;
  } else {
    return 0;
  }
}

void cleanup_module (void) {
  if (MOD_IN_USE)
    printk ("Vproc: device busy, remove delayed.\n");
  if (unregister_chrdev (VPROC_MAJOR, "vproc") != 0) {
    printk ("cleanup_module failed.\n");
  } else {
    printk ("cleanup_module succeeded.\n");
  }
}

#ifdef __cplusplus
}
#endif


/*
struct file_operations {
        int (*lseek) (struct inode *, struct file *, off_t, int);
        int (*read) (struct inode *, struct file *, char *, int);
        int (*write) (struct inode *, struct file *, const char *, int);
        int (*readdir) (struct inode *, struct file *, void *, filldir_t);
        int (*select) (struct inode *, struct file *, int, select_table *);
        int (*ioctl) (struct inode *, struct file*, unsigned int, unsigned long);
        int (*mmap) (struct inode *, struct file *, struct vm_area_struct *);
        int (*open) (struct inode *, struct file *);
        void (*release) (struct inode *, struct file *);
        int (*fsync) (struct inode *, struct file *);
        int (*fasync) (struct inode *, struct file *, int);
        int (*check_media_change) (kdev_t dev);
        int (*revalidate) (kdev_t dev);
};
*/
