/*-----------------------------------------------------------*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <math.h>
/*-----------------------------------------------------------*/

#ifndef byte
  typedef unsigned char byte;
#endif
#ifndef word
  typedef unsigned short word;
#endif

/*static*/ /* word          VxP_BASE; */
static unsigned int VxP_MEMWIN;
static void*        VxP_MEMBUFFER;
static word          VxP_MEMWINSIZE;

#include "vxp202.h"

/*-----------------------------------------------------------*/

static int dd;

int VxP_Init() {
  dd=open("/dev/vproc",O_RDWR);
  if (dd<0)
    return 0;
  return dd;
}

byte VxP_PortRead(word port) {
  byte data;
  off_t n;

  n=lseek(dd,(off_t)port,SEEK_SET);
  read(dd,&data,sizeof(data));
  return data;
}

void VxP_PortWrite(word port,byte data) {
  int n;
  off_t o;

  o=lseek(dd,(off_t)port,SEEK_SET);
  n=write(dd,&data,sizeof(data));
  /*  printf("  (%02x) := %02x (=%02x)\n",o,data,n);*/
}

void VxP_Register() {
  word i;
  printf("\n    ");
  for (i=0;i<16;i++)
    printf("%02x ",i);
  printf("\n");
  for (i=0;i<256;i++) {
    if (!(i % 16))
      printf("\n%02x  ",i);
    printf("%02x ",VxP_PortRead(i));
  }
  printf("\n");
}

void VxP_Reset() {
  VxP_PortWrite(VxP_RESET,0xCF);        /* reset VxP202 */
  VxP_PortWrite(VxP_OUT_PROC_CTRL_B,0x00);
  #ifdef DOS
    delay(100);
  #endif
  VxP_PortWrite(VxP_RESET,0x0c);        /* reset VxP202 */
  VxP_PortWrite(VxP_OUT_PROC_CTRL_B,VxP_VIDEO_OUT_ENABLE);
}

void VxP_SetIOBase(word vxpbase) {
  /*  VxP_BASE=vxpbase; */
  VxP_PortWrite(VxP_IO_BASE,(VxP_BASE >> 2) & 0xff);
}

void VxP_SetMemWin(unsigned int memwin,word size) {
  VxP_MEMWIN    =memwin;
  VxP_MEMWINSIZE=size;
  /* set adress, window size, and disable it */
  if (size==128)
    VxP_PortWrite(VxP_MEM_WIN_CTRL,VxP_MEMWIN >> 11 | VxP_MEM_WIN_128K);
  else {
    VxP_MEMWINSIZE=32;
    VxP_PortWrite(VxP_MEM_WIN_CTRL,VxP_MEMWIN >> 11 | VxP_MEM_WIN_128K);
  }
}

void VxP_SetISAWinSize(word width,word height) {
  VxP_PortWrite(VxP_ISA_SRC_WIDTH_A ,(width-1) % 256);
  VxP_PortWrite(VxP_ISA_SRC_WIDTH_B ,(width-1) >> 8);
  VxP_PortWrite(VxP_ISA_SRC_HEIGHT_A,(height-1) % 256);
  VxP_PortWrite(VxP_ISA_SRC_HEIGHT_B,(height-1) >> 8);
}

static word CROP_LEFT,CROP_RIGHT,CROP_TOP,CROP_BOTTOM;

void VxP_SetCropping(word left,word right,word top,word bottom) {
  VxP_PortWrite(VxP_HOR_CROP_LEFT_A  ,left % 256);
  VxP_PortWrite(VxP_HOR_CROP_LEFT_B  ,left >> 8);
  VxP_PortWrite(VxP_HOR_CROP_RIGHT_A ,right % 256);
  VxP_PortWrite(VxP_HOR_CROP_RIGHT_B ,right >> 8);
  VxP_PortWrite(VxP_VER_CROP_TOP_A   ,top %256);
  VxP_PortWrite(VxP_VER_CROP_TOP_B   ,top >> 8);
  VxP_PortWrite(VxP_VER_CROP_BOTTOM_A,bottom % 256);
  VxP_PortWrite(VxP_VER_CROP_BOTTOM_B,bottom >> 8);
  CROP_LEFT  =left;
  CROP_RIGHT =right;
  CROP_TOP   =top;
  CROP_BOTTOM=bottom;
}

void VxP_SetScaling(float h,float v) {
  word horizontal,vertical;

  horizontal=(word)(1023.0*h);
  vertical=(word)(1023.0*v);
  VxP_PortWrite(VxP_HOR_SCALE_CTRL_A,(horizontal << 6) & 0xc0);
  VxP_PortWrite(VxP_HOR_SCALE_CTRL_B,horizontal >> 2);
  VxP_PortWrite(VxP_VER_SCALE_CTRL_A,(vertical << 6) & 0xc0);
  VxP_PortWrite(VxP_VER_SCALE_CTRL_B,vertical >> 2);
}

void VxP_SetZoom(float h,float v) {
  word horizontal,vertical;

  horizontal=(word)(2047.0*h);
  vertical=(word)(2047.0*v);
  VxP_PortWrite(VxP_OUT_VZOOM_CTRL_A,vertical % 256);
  VxP_PortWrite(VxP_OUT_VZOOM_CTRL_B,vertical >> 8);
  VxP_PortWrite(VxP_OUT_HZOOM_CTRL_A,horizontal % 256);
  VxP_PortWrite(VxP_OUT_HZOOM_CTRL_B,horizontal >> 8);
}

void VxP_LockViewPort() {
  byte b;

//  b=VxP_PortRead(VxP_OUT_PROC_CTRL_B);
//  VxP_PortWrite(VxP_OUT_PROC_CTRL_B,b & ((!VxP_VPORT_REG_LOAD_EN) & 0xff));
}

void VxP_UnlockViewPort() {
  byte b;

//  b=VxP_PortRead(VxP_OUT_PROC_CTRL_B);
//  VxP_PortWrite(VxP_OUT_PROC_CTRL_B,b | VxP_VPORT_REG_LOAD_EN);
}

void VxP_SetViewAdress(word adr) {
  byte tmp;

  VxP_LockViewPort();
  VxP_PortWrite(VxP_VPORT_START_ADR_A,adr % 256);
  tmp=VxP_PortRead(VxP_VPORT_ADR_WH_B);
  tmp=(tmp & 0x3f) | ((adr >> 2) & 0xc0);
  VxP_PortWrite(VxP_VPORT_ADR_WH_B,tmp);
  VxP_UnlockViewPort();
}

void VxP_SetViewPortOrigin(word left,word top) {
  word ofsx,ofsy;

  ofsx=VxP_PortRead(VxP_HOR_COORD_OFS);
  ofsy=VxP_PortRead(VxP_VER_COORD_OFS);

  left  =left-CROP_LEFT+ofsx;
  top   =ofsy-CROP_TOP+top;

  VxP_LockViewPort();
  VxP_PortWrite(VxP_VPORT_DPY_OR_TOP_A,top % 256);
  VxP_PortWrite(VxP_VPORT_DPY_OR_L_A,left % 256);
  VxP_PortWrite(VxP_VPORT_DPY_OR_TL_B,((top >> 2) & 0xc0) | (left >> 8));
  VxP_UnlockViewPort();
}

void VxP_SetViewPortSize(word width,word height) {
  byte tmp;

  VxP_LockViewPort();
  VxP_PortWrite(VxP_VPORT_WIDTH_A,width % 256);
  VxP_PortWrite(VxP_VPORT_HEIGHT_A,height % 256);
  tmp=VxP_PortRead(VxP_VPORT_ADR_WH_B);
  tmp=(tmp & 0xc0) | ((width >> 4) & 0x10) | (height >> 8);
  VxP_PortWrite(VxP_VPORT_ADR_WH_B,tmp);
  VxP_UnlockViewPort();
}

void VxP_SetDisplayWindow(word left,word top,word width,word height) {
  word right,bottom;
  word ofsx,ofsy;
  word cropx,cropy;

  ofsx=VxP_PortRead(VxP_HOR_COORD_OFS);
  ofsy=VxP_PortRead(VxP_VER_COORD_OFS);

  left  =left+ofsx+CROP_LEFT;
  right =left+width-CROP_RIGHT;
  top   =top+ofsy+CROP_TOP;
  bottom=top+height-CROP_BOTTOM;

  VxP_PortWrite(VxP_DPY_WIN_LEFT_A,left & 256);
  VxP_PortWrite(VxP_DPY_WIN_RIGHT_A,right & 256);
  VxP_PortWrite(VxP_DPY_WIN_LR_B,((left >> 3) & 0xe0) | ((right >> 8) & 0x07));

  VxP_PortWrite(VxP_DPY_WIN_TOP_A,top & 256);
  VxP_PortWrite(VxP_DPY_WIN_BOTTOM_A,bottom & 256);
  VxP_PortWrite(VxP_DPY_WIN_TB_B,((top >> 2) & 0xc0) | ((bottom >> 8) & 0x03));
}

void VxP_SetColorKey24(byte r,byte g,byte b) {
  VxP_PortWrite(VxP_KEY_COLOR,r);
  VxP_PortWrite(VxP_KEY_COLOR_HI,g);
  VxP_PortWrite(VxP_KEY_COLOR_HI2,b);
}

void VxP_SetColorKey16(byte r,byte g,byte b) {
  VxP_SetColorKey24(0,0,0);
}

void VxP_SetColorKey8(byte colorkey) {
  VxP_SetColorKey24(0,0,colorkey);
}

void VxP_SetCoordinateOffset(byte horizontal,byte vertical) {
  VxP_PortWrite(VxP_VER_COORD_OFS,vertical);
  VxP_PortWrite(VxP_HOR_COORD_OFS,horizontal);
}

void VxP_SetFramesPerSecond(byte fps) {
  VxP_PortWrite(VxP_TIME_SCALE_CTRL,fps);
}

void VxP_Initialize(word vxpbase,
                    unsigned int memwin,word memwinsize) {
  word i;

  printf("VxP202 low level driver, (C) Nils Schmeisser, 1997\n\n");

  /*--- base configuration --------------------------------------*/

  VxP_Init();
  VxP_SetIOBase(vxpbase);
  printf("VxP202: I/O base adress = %x\n",
         (VxP_PortRead(VxP_IO_BASE) | 0x100) << 2);

  VxP_Reset();
  /* disable memory window */
  VxP_PortWrite(VxP_MEM_WIN_CTRL,VxP_MEM_WIN_DISABLE);
  /* dRAM settings at this point, is this fixed for all ESCOM releases? */
  VxP_PortWrite(VxP_DRAM_CTRL,0xda);
  /* ISA ctrl, we use 16bit dRAM */
  VxP_PortWrite(VxP_ISA_CTRL,0x02);

  /*--- video input configuration -------------------------------*/

  /* input video configuration */
  VxP_PortWrite(VxP_INPUT_CFG_A,0x1a); /* reset video input */
  /* video decoder enabled */
  VxP_PortWrite(VxP_INPUT_CFG_A,0x08);
  /* YUV 422, compressed, progressive input, 8bit 422 differential, VIO */
  VxP_PortWrite(VxP_INPUT_CFG_B,0x5c);
  /* Vin multi buffer depth A = 0x82, what ever this means */
  VxP_PortWrite(VxP_VIN_MBUF_DEPTH_A,0x82);
  /* capture all frames */
  VxP_SetFramesPerSecond(0);
  /* video buffer layout, use 512 lines */
  VxP_PortWrite(VxP_VIDEO_LAYOUT_CTRL,VxP_VIDEO_BUFFER_1024);
  /* set input fifo to 0x52 ??? */
  VxP_PortWrite(VxP_INPUT_FIFO,0x52);
  /* disable capturing */
  VxP_PortWrite(VxP_CAPT_CTRL,VxP_CAPT_REG_HOLD);
  /* no filtering */
  VxP_PortWrite(VxP_FILTER_CTRL,0x00);

  /*--- video output configuration -------------------------------*/
  VxP_PortWrite(VxP_OUT_PROC_CTRL_A,VxP_HZOOM_ENABLE | VxP_HINTERPOL_ENABLE);
  VxP_PortWrite(VxP_OUT_PROC_CTRL_B,VxP_VIDEO_OUT_ENABLE
                | VxP_VZOOM_ENABLE | VxP_VZOOM_IP_ENABLE | VxP_VPORT_REG_LOAD_EN);
  /* display control */
  VxP_PortWrite(VxP_DISPLAY_CTRL,/* VxP_VIDEO_DPY_ONLY*/ VxP_VIDEO_KEY_DPY
                | VxP_VGA_WINDOW_ENABLE | VxP_VGA_COL_KEY_ENABLE);

//  VxP_SetCoordinateOffset(120,120);

  /* cropping */
  VxP_SetCropping(10,10,20,20);

//  VxP_SetCoordinateOffset(0,100);
  VxP_SetDisplayWindow(80,80,301,211);
  VxP_SetColorKey24(0x01,0x01,0x01);
//  VxP_SetColorKey8(0xfd);

  /* scaling disabled */
  VxP_SetScaling(0.5,0.5);
  VxP_SetZoom(2,2);

  VxP_SetViewAdress(0x0);
  VxP_SetViewPortOrigin(80,80);
  VxP_SetViewPortSize(300,210);
//  VxP_FlushViewPort();

  /* output fifo threshold ??? */
  VxP_PortWrite(0x90,0x3b);
  /* out req active level ??? */
  VxP_PortWrite(0x91,0x30);
  /* multi buffer depth control ??? */
  VxP_PortWrite(0x93,0x01);
  /* VGA control */
  VxP_PortWrite(VxP_VGA_CTRL_A,VxP_VGA_NON_INTERLACED);
//  VxP_PortWrite(VxP_VGA_CTRL_B,VxP_VGA_CLK_3 | VxP_VGA_24BIT_DISABLE);
  /* output processing control */
  VxP_PortWrite(VxP_OUT_PROC_CTRL_C,0x00);
  /* VIDEON control ??? */
  VxP_PortWrite(VxP_VIDEON_CTRL,0x00);


}
