#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <math.h>

#ifndef word
  typedef unsigned int word;
#endif
#ifndef byte
  typedef unsigned char byte;
#endif

static void *VxP_MEMWIN;
static word VxP_MEMWINSIZE;
static word VxP_PORT;

#include "vproc.h"
#include "vxp202.h"

static int dd;

int VxP_Init() {
  dd=open("/dev/vproc",O_RDWR);
  if (dd<0)
    return 0;
  return dd;
}

byte VxP_PortRead(word port) {
  byte data;
  off_t n;

  n=lseek(dd,(off_t)port,SEEK_SET);
  read(dd,&data,sizeof(data));
  return data;
}

void VxP_PortWrite(word port,byte data) {
  int n;
  off_t o;

  o=lseek(dd,(off_t)port,SEEK_SET);
  n=write(dd,&data,sizeof(data));
  printf("  (%02x) := %02x (=%02x)\n",o,data,n);
}

void VxP_Register() {
  word i;
  byte j;

  printf("    ");
  for (i=0;i<16;i++)
    printf("%02x ",i);
  printf("\n");

  for (i=0;i<256;i++) {
    if (i % 16 == 0) printf("%02x  ",i);
    j=VxP_PortRead(i);
    printf("%02x ",j);
    if (i % 16 == 15) printf("\n");
  }
}

void VxP_Done() { close(dd); }

/* for now no interrupt driven I2C IO */

/* times in ns */
#define I2C_DLY_FALL    300  /* maximum falling time */
#define I2C_DLY_RAISE  1000  /* maximum raise time */
#define I2C_DLY_BUF    4700  /* bus free time between start and stop */
#define I2C_DLY_HDSTA  4000  /* hold time start period */
#define I2C_DLY_SUSTO  4000  /* setup time stop condition */
#define I2C_DLY_SUDAT   250  /* data setup time */
#define I2C_DLY_LOW    4700  /* low period of SCL clock */
#define I2C_DLY_HIGH   4000  /* high period of SCL clock */

static float I2C_DLY;
static int I2C_PORT;

/* delay for dly ns */

void I2CWait(unsigned int dly) {
  unsigned long i,n;

  n=(unsigned long)(I2C_DLY*(float)dly)+1;
  for (i=0;i<n;i++);
}

int I2CGetSDA() {
  return I2C_PORT & VxP_I2C_DATA;
}

int I2CGetSCL() {
  return I2C_PORT & VxP_I2C_CLK;
}

void I2CSetSDA(int lh) {
  #ifdef VxP202_
    if (lh) {
      I2C_PORT = I2C_PORT | VxP_I2C_DATA;
    } else {
      I2C_PORT = I2C_PORT & VxP_I2C_CLK;
    }
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
  #endif
}

void I2CSetSCL(int lh) {
  #ifdef VxP202_
    if (lh) {
      I2C_PORT = I2C_PORT | VxP_I2C_CLK;
    } else {
      I2C_PORT = I2C_PORT & VxP_I2C_DATA;
    }
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
  #endif
}

void I2CStart() {
  /*    ____
            |_____
        __________
                    CLCK
  */
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_HDSTA);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
  /* now ready to transmit data */
}

void I2CStop() {
  /*
          __
     ____|
     _______
             CLCK
  */
  /* we assume that both SDA and SCL are low */
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_SUSTO);
  I2CSetSDA(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_BUF);
  /* transmission completed */
}

void I2CHigh() {
  /*
      _______ H
              L
         _
      __| |__ CLK
  */
  /* we assume that both SDA and SCL are low */
  I2CSetSDA(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_HIGH);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL);
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
}

void I2CLow() {
  /*
              H
      _______ L
         _
      __| |__ CLK
  */
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE+I2C_DLY_HIGH);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
}

int I2CAck() {
  word value;
  /*
      _________ H
                L
         ___
      __|   |__ CLK

          ^
          | read ACK / data
  */
  I2CSetSDA(1);
  I2CWait(I2C_DLY_FALL+I2C_DLY_SUDAT);
  I2CSetSCL(1);
  I2CWait(I2C_DLY_RAISE);
  value=VxP_PortRead(VxP_I2C_CTRL);
  I2CSetSCL(0);
  I2CWait(I2C_DLY_FALL);
  I2CSetSDA(0);
  I2CWait(I2C_DLY_FALL+I2C_DLY_LOW);
  return value;
}

int I2CSendByte(int data) {
  int i;

  for (i=0;i<8;i++) {
    if (data & (1 << i))
      I2CHigh();
    else
      I2CLow();
  }
  i=I2CAck();
  if (!i)
    return 0;
  return 1;
}

static int I2CReadByte(void) {
  int i;
  int data=0;

  /*
      _________ H
                L
         ___
      __|   |__ CLK

          ^
          | read data
  */
  /* set data output bit to H, CLK L */
  return data;
}

void I2CInit() {
  clock_t t1,t2,dt;
  unsigned long i,cnt;

  #ifdef VxP202_
    VxP_PortWrite(0x37,0);
    I2C_PORT = VxP_I2C_CLK | VxP_I2C_DATA | VxP_I2C_FRAME;
    VxP_PortWrite(VxP_I2C_CTRL,I2C_PORT);
    I2CWait(2*I2C_DLY_BUF);
  #endif

  cnt=1;
  do {
    cnt=cnt*2;
    t1=clock();
    for (i=0;i<cnt;i++);
    t2=clock();
    dt=t2-t1;
  } while (dt<20);
  printf("dt=%d cnt=%f\n",dt,(float)cnt);
  #ifdef DOS
    I2C_DLY=((float)cnt/1e9)*((float)CLK_TCK/(float)dt);
  #else
    I2C_DLY=((float)cnt/1e9)*((float)CLOCKS_PER_SEC/(float)dt);
  #endif
  printf("I2C_DLY=%f\n",I2C_DLY);
}

static void scan_i2c(void) {
  int i;

  for (i=0; i<256; i++) {
    printf("scanning %02x ",i);
    I2CStart();
    if (I2CSendByte(i))
      printf(" ACK");
    I2CStop();
    printf("\n");
  }
}

void VxP_Reset() {
  VxP_PortWrite(VxP_RESET,0xCF);        /* reset VxP202 */
}

void VxP_SetIOBase(word vxpbase) {
  VxP_PORT=vxpbase;
  VxP_PortWrite(VxP_IO_BASE,(VxP_PORT >> 2) & (byte)0xff);
}

void VxP_SetMemWin(void *memwin,word size) {
  VxP_MEMWIN    =memwin;
  VxP_MEMWINSIZE=size;
  /* set adress, window size, and disable it */
  if (size==128)
    VxP_PortWrite(VxP_MEM_WIN_CTRL,
		  (word)((unsigned long)VxP_MEMWIN >> 11) | VxP_MEM_WIN_128K);
  else {
    VxP_MEMWINSIZE=32;
    VxP_PortWrite(VxP_MEM_WIN_CTRL,
		  (word)((unsigned long)VxP_MEMWIN >> 11) | VxP_MEM_WIN_128K);
  }
}

void VxP_SetISAWinSize(int width,int height) {
  VxP_PortWrite(VxP_ISA_SRC_WIDTH_A ,(width-1) % 256);
  VxP_PortWrite(VxP_ISA_SRC_WIDTH_B ,(width-1) >> 8);
  VxP_PortWrite(VxP_ISA_SRC_HEIGHT_A,(height-1) % 256);
  VxP_PortWrite(VxP_ISA_SRC_HEIGHT_B,(height-1) >> 8);
}

void VxP_SetCropping(int left,int right,int top,int bottom) {
  VxP_PortWrite(VxP_HOR_CROP_LEFT_A  ,left % 256);
  VxP_PortWrite(VxP_HOR_CROP_LEFT_B  ,left >> 8);
  VxP_PortWrite(VxP_HOR_CROP_RIGHT_A ,right % 256);
  VxP_PortWrite(VxP_HOR_CROP_RIGHT_B ,right >> 8);
  VxP_PortWrite(VxP_VER_CROP_TOP_A   ,top %256);
  VxP_PortWrite(VxP_VER_CROP_TOP_B   ,top >> 8);
  VxP_PortWrite(VxP_VER_CROP_BOTTOM_A,bottom % 256);
  VxP_PortWrite(VxP_VER_CROP_BOTTOM_B,bottom >> 8);
}

void VxP_Initialize(word vxpbase,
                    void *memwin,int memwinsize) {
  int i;
  /*  char *icode="AURAV1"; */

  printf("VxP202 low level driver, (C) Nils Schmeisser, 1997\n\n");

  VxP_SetIOBase(vxpbase);
  printf("VxP202: I/O base adress = %x\n",
         (VxP_PortRead(VxP_IO_BASE) | 0x100) << 2);
  VxP_Reset();


  /* we should alloc nemwinsize here
    VxP_MEMBUFFER=malloc(memwinsize);
  */
  VxP_SetMemWin(memwin,memwinsize);
  i=VxP_PortRead(VxP_MEM_WIN_CTRL);
  printf("VxP202: memory window = %x, size = %d\n",
         (i & 0x1f) << 11,(i & 0x40) << 1);

  VxP_PortWrite(VxP_INPUT_CFG_A,0x1a); /* reset video input */
  VxP_PortWrite(VxP_INPUT_CFG_A,0x0a); /* reset video input */
  VxP_PortWrite(VxP_INPUT_CFG_B,0x54);

  VxP_SetISAWinSize(160,120);       /* ISA win with ratio 4:3 */
  VxP_SetCropping(0,1023,0,1023);

  /*  VxP_PortWrite(VxP_FILTER_CTRL,0); now filtering */

  /*
  #define VxP_HOR_SCALE_CTRL_A   0x54
  #define VxP_HOR_SCALE_CTRL_B   0x55
  #define VxP_VER_SCALE_CTRL_A   0x5B
  #define VxP_VER_SCALE_CTRL_B   0x5C
  #define VxP_PICT_CTRL_ADR      0x5D
  #define VxP_PICT_CTRL_LUMA     0x5E
  #define VxP_PICT_CTRL          0x5F
  VxP_PortWrite(VxP_HOR_SCALE_CTRL_B,0x20);
  */

  /*  VxP_PortWrite(VxP_TIME_SCALE_CTRL,0xf0); */

}



main(int argc,char **argv) {
  word wport,wdata,i;
  void *memwin;

  if (VxP_Init()) {
    memwin=(void*)malloc(32*1024);
    VxP_Initialize(0x500,memwin,32);

    /*    VxP_Register();
	  
	  printf("scanning for I2C devices\n");
	  I2CInit();
	  scan_i2c(); */

    do {
      printf("port data:");
      scanf("%x %x",&wport,&wdata);
      VxP_PortWrite(wport,wdata);
    } while (wport>0);

    /*
      I2CInit();
      scan_i2c();
      */

    VxP_Done();
  } else
    printf("failed open /dev/vproc\n");
}

