/*-----------------------------------------------------------*/
#include <stdio.h>
#include <asm/io.h>

/*-----------------------------------------------------------*/

static int          VxP_BASE;
static unsigned int VxP_MEMWIN;
static void*        VxP_MEMBUFFER;
static int          VxP_MEMWINSIZE;

#include "vxp202.h"

/*-----------------------------------------------------------*/

/* #define DOS
 */

#ifdef DOS
  #include <dos.h>
  #define outb(PORT,DATA) outportb(PORT,DATA)
  #define outw(PORT,DATA) outport(PORT,DATA)
  #define inb(PORT) inportb(PORT)
  #define inw(PORT) inport(PORT)
#endif

/*-----------------------------------------------------------*/

int VxP_PortRead(int port) {
  outw(VxP_INDEX,port);
  return inb(VxP_DATA_READ);
}

void VxP_InputPipelineLock() {
  int b;
  outw(VxP_INDEX,VxP_INPUT_CFG_A);
  b=inb(VxP_DATA_READ);
  outw(VxP_DATA_WRITE,b | VxP_INPUT_PIPE_RESET);
}

void VxP_InputPipelineUnlock() {
  int b;
  outw(VxP_INDEX,VxP_INPUT_CFG_A);
  b=inb(VxP_DATA_READ);
  outw(VxP_DATA_WRITE,b & !VxP_INPUT_PIPE_RESET);
}

void VxP_PortWrite(int port,int data) {
  VxP_InputPipelineLock();
  outw(VxP_INDEX,port);
  outb(VxP_DATA_WRITE,data);
  VxP_InputPipelineUnlock();
  printf("VxP (%02x):=%02x -> =%02x\n",port,data,VxP_PortRead(port));
}

void VxP_Register() {
  int i;
  printf("\n    ");
  for (i=0;i<16;i++)
    printf("%02x ",i);
  printf("\n");
  for (i=0;i<256;i++) {
    if (!(i % 16))
      printf("\n%02x  ",i);
    printf("%02x ",VxP_PortRead(i));
  }
  printf("\n");
}

void VxP_Reset() {
  VxP_PortWrite(VxP_RESET,0xCF);        /* reset VxP202 */
}

void VxP_SetIOBase(int vxpbase) {
  VxP_BASE=vxpbase;
  VxP_PortWrite(VxP_IO_BASE,(VxP_BASE >> 2) & 0xff);
}

void VxP_SetMemWin(unsigned int memwin,int size) {
  VxP_MEMWIN    =memwin;
  VxP_MEMWINSIZE=size;
  /* set adress, window size, and disable it */
  if (size==128)
    VxP_PortWrite(VxP_MEM_WIN_CTRL,VxP_MEMWIN >> 11 | VxP_MEM_WIN_128K);
  else {
    VxP_MEMWINSIZE=32;
    VxP_PortWrite(VxP_MEM_WIN_CTRL,VxP_MEMWIN >> 11 | VxP_MEM_WIN_128K);
  }
}

void VxP_SetISAWinSize(int width,int height) {
  VxP_PortWrite(VxP_ISA_SRC_WIDTH_A ,(width-1) % 256);
  VxP_PortWrite(VxP_ISA_SRC_WIDTH_B ,(width-1) >> 8);
  VxP_PortWrite(VxP_ISA_SRC_HEIGHT_A,(height-1) % 256);
  VxP_PortWrite(VxP_ISA_SRC_HEIGHT_B,(height-1) >> 8);
}

void VxP_SetCropping(int left,int right,int top,int bottom) {
  VxP_PortWrite(VxP_HOR_CROP_LEFT_A  ,left % 256);
  VxP_PortWrite(VxP_HOR_CROP_LEFT_B  ,left >> 8);
  VxP_PortWrite(VxP_HOR_CROP_RIGHT_A ,right % 256);
  VxP_PortWrite(VxP_HOR_CROP_RIGHT_B ,right >> 8);
  VxP_PortWrite(VxP_VER_CROP_TOP_A   ,top %256);
  VxP_PortWrite(VxP_VER_CROP_TOP_B   ,top >> 8);
  VxP_PortWrite(VxP_VER_CROP_BOTTOM_A,bottom % 256);
  VxP_PortWrite(VxP_VER_CROP_BOTTOM_B,bottom >> 8);
}

void VxP_Initialize(int vxpbase,
                    unsigned int memwin,int memwinsize) {
  int i;
//  char *icode="AURAV1";

  printf("VxP202 low level driver, (C) Nils Schmeisser, 1997\n\n");

  VxP_SetIOBase(vxpbase);
  printf("VxP202: I/O base adress = %x\n",
         (VxP_PortRead(VxP_IO_BASE) | 0x100) << 2);

  VxP_Reset();

  /* we should alloc nemwinsize here
    VxP_MEMBUFFER=malloc(memwinsize);
  */
  VxP_SetMemWin(memwin,memwinsize);
  i=VxP_PortRead(VxP_MEM_WIN_CTRL);
  printf("VxP202: memory window = %x, size = %d\n",
         (i & 0x1f) << 11,(i & 0x40) << 1);

  VxP_PortWrite(VxP_INPUT_CFG_A,0x1a); /* reset video input */
  VxP_PortWrite(VxP_INPUT_CFG_A,0x0a); /* reset video input */
  VxP_PortWrite(VxP_INPUT_CFG_B,0x54);

  VxP_SetISAWinSize(160,120);       /* ISA win with ratio 4:3 */
  VxP_SetCropping(0,1023,0,1023);

//  VxP_PortWrite(VxP_FILTER_CTRL,0); /* now filtering */

  /*
  #define VxP_HOR_SCALE_CTRL_A   0x54
  #define VxP_HOR_SCALE_CTRL_B   0x55
  #define VxP_VER_SCALE_CTRL_A   0x5B
  #define VxP_VER_SCALE_CTRL_B   0x5C
  #define VxP_PICT_CTRL_ADR      0x5D
  #define VxP_PICT_CTRL_LUMA     0x5E
  #define VxP_PICT_CTRL          0x5F
  VxP_PortWrite(VxP_HOR_SCALE_CTRL_B,0x20);
  */


//  VxP_PortWrite(VxP_TIME_SCALE_CTRL,0xf0);

  VxP_Register();

}
